@@ -29,12 +29,21 @@ __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import inspect
-import time
 import traceback
 import os
-import pyatspi
+import re
 import subprocess
 import sys
+import types
+
+from datetime import datetime
+
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
+
+from .ax_object import AXObject
+from .ax_utilities import AXUtilities
 
 # Used to turn off all debugging.
 #
@@ -154,11 +163,11 @@ TRACE_APPS = []
 #
 TRACE_EVENTS = []
 
-# What pyatspi role(s) should be traced if traceit is being used. By
+# What role(s) should be traced if traceit is being used. By
 # default, we'll trace everything. An example of what you might wish
 # to do to narrow things down, if you know buttons trigger the problem:
 #
-# TRACE_ROLES = [pyatspi.ROLE_PUSH_BUTTON, pyatspi.ROLE_TOGGLE_BUTTON]
+# TRACE_ROLES = [Atspi.Role.PUSH_BUTTON, Atspi.Role.TOGGLE_BUTTON]
 #
 TRACE_ROLES = []
 
@@ -194,7 +203,81 @@ def printStack(level):
         traceback.print_stack(None, 100, debugFile)
         println(level)
 
-def println(level, text="", timestamp=False):
+def _asString(obj):
+    if isinstance(obj, Atspi.Accessible):
+        result = AXObject.get_role_name(obj)
+        name = AXObject.get_name(obj)
+        if name:
+            result += f": '{name}'"
+        if not result:
+            result = "DEAD"
+
+        return f"[{result}]"
+
+    if isinstance(obj, Atspi.Event):
+        return (
+            f"{obj.type} for {_asString(obj.source)} in "
+            f"{_asString(AXObject.get_application(obj.source))} "
+            f"({obj.detail1}, {obj.detail2}, {_asString(obj.any_data)})"
+        )
+
+    if isinstance(obj, (Atspi.Role, Atspi.StateType, Atspi.CollectionMatchType)):
+        return obj.value_nick
+
+    if isinstance(obj, list):
+        return f"[{', '.join(map(_asString, obj))}]"
+
+    if isinstance(obj, types.FunctionType):
+        if hasattr(obj, "__self__"):
+            return f"{obj.__module__}.{obj.__self__.__class__.__name__}.{obj.__name__}"
+        return f"{obj.__module__}.{obj.__name__}"
+
+    if isinstance(obj, types.FrameType):
+        module_name = inspect.getmodulename(obj.f_code.co_filename)
+        return f"{module_name}.{obj.f_code.co_name}"
+
+    if isinstance(obj, inspect.FrameInfo):
+        module_name = inspect.getmodulename(obj.filename)
+        return f"{module_name}.{obj.function}"
+
+    return str(obj)
+
+def printTokens(level, tokens, timestamp=False, stack=False):
+    if level < debugLevel:
+        return
+
+    text = " ".join(map(_asString, tokens))
+    text = re.sub(r"[ \u00A0]+", " ", text)
+    text = re.sub(r" (?=[,.:)])(?![\n])", "", text)
+    println(level, text, timestamp, stack)
+
+def printMessage(level, text, timestamp=False, stack=False):
+    if level < debugLevel:
+        return
+
+    println(level, text, timestamp, stack)
+
+def _stackAsString(max_frames=4):
+    callers = []
+    current_module = inspect.getmodule(inspect.currentframe())
+    stack = inspect.stack()
+    for i in range(1, len(stack)):
+        frame = stack[i]
+        module = inspect.getmodule(frame[0])
+        if module == current_module:
+            continue
+        if frame.function == 'main':
+            continue
+        if module is None or module.__name__ is None:
+            continue
+        callers.append(frame)
+        if len(callers) >= max_frames:
+            break
+
+    callers.reverse()
+    return " > ".join(map(_asString, callers))
+
+def println(level, text="", timestamp=False, stack=False):
     """Prints the text to stderr unless debug is enabled.
 
     If debug is enabled the text will be redirected to the
@@ -208,14 +291,18 @@ def println(level, text="", timestamp=False):
     if level >= debugLevel:
         text = text.replace("\ufffc", "[OBJ]")
         if timestamp:
-            text = "%s - %s" % (time.strftime("%H:%M:%S"), text)
+            text = text.replace("\n", f"\n{' ' * 18}")
+            text = f"{datetime.now().strftime('%H:%M:%S.%f')} - {text}"
+        if stack:
+            text += f" {_stackAsString()}"
+
         if debugFile:
             try:
                 debugFile.writelines([text, "\n"])
             except TypeError:
                 text = "TypeError when trying to write text"
                 debugFile.writelines([text, "\n"])
-            except:
+            except Exception:
                 text = "Exception when trying to write text"
                 debugFile.writelines([text, "\n"])
         else:
@@ -224,7 +311,7 @@ def println(level, text="", timestamp=False):
             except TypeError:
                 text = "TypeError when trying to write text"
                 sys.stderr.writelines([text, "\n"])
-            except:
+            except Exception:
                 text = "Exception when trying to write text"
                 sys.stderr.writelines([text, "\n"])
 
@@ -248,8 +335,8 @@ def printResult(level, result=None):
     callString = 'CALL:   %s.%s (line %s) -> %s.%s%s' % (
         inspect.getmodulename(prev[1]), prev[3], prev[2],
         inspect.getmodulename(current[1]), current[3], fArgs)
-    string = '%s\n%s %s' % (callString, 'RESULT:', result)
-    println(level, '%s' % string)
+    string = f'{callString}\nRESULT: {result}'
+    println(level, f'{string}')
 
 def printObjectEvent(level, event, sourceInfo=None, timestamp=False):
     """Prints out an Python Event object.  The given level may be
@@ -268,12 +355,16 @@ def printObjectEvent(level, event, sourceInfo=None, timestamp=False):
 
     level = max(level, eventDebugLevel)
 
+    anydata = event.any_data
+    if isinstance(anydata, str) and len(anydata) > 100:
+        anydata = f"{anydata[0:100]} (...)"
+
     text = "OBJECT EVENT: %s (%d, %d, %s)" \
-           % (event.type, event.detail1, event.detail2, event.any_data)
+           % (event.type, event.detail1, event.detail2, anydata)
     println(level, text, timestamp)
 
     if sourceInfo:
-        println(level, "             %s" % sourceInfo, timestamp)
+        println(level, f"{' ' * 18}{sourceInfo}", timestamp)
 
 def printInputEvent(level, string, timestamp=False):
     """Prints out an input event.  The given level may be overridden
@@ -316,67 +407,30 @@ def getAccessibleDetails(level, acc, indent="", includeApp=True):
         return ""
 
     if includeApp:
-        try:
-            app = acc.getApplication()
-        except:
-            string = indent + "app=(exception getting app) "
-            app = None
-        else:
-            if app:
-                try:
-                    string = indent + "app.name='%s' " % app.name
-                except (LookupError, RuntimeError):
-                    string = indent + "app.name='(exception getting name)' "
-            else:
-                string = indent + "app=None "
+        string = indent + f"app='{AXObject.application_as_string(acc)}' "
     else:
         string = indent
 
-    # create the States string
-    try:
-        stateSet = acc.getState()
-    except:
-        string += "(exception getting state set)"
-    try:
-        states = stateSet.getStates()
-    except:
-        string += "(exception getting states)"
-        states = []
-    state_strings = []
-    for state in states:
-        state_strings.append(pyatspi.stateToString(state))
-    state_string = ' '.join(state_strings)
-
-    # create the relations string
-    try:
-        relations = acc.getRelationSet()
-    except:
-        string += "(exception getting relation set)"
-        relations = None
-    if relations:
-        relation_strings = []
-        for relation in relations:
-            relation_strings.append( \
-                          pyatspi.relationToString(relation.getRelationType()))
-        rel_string = ' '.join(relation_strings)
-    else:
-        rel_string = ''
-
-    try:
-        iface_string = " ".join(pyatspi.utils.listInterfaces(acc))
-    except:
-        iface_string = "(exception calling listInterfaces)"
-
-    try:
-        string += "name='%s' role='%s' state='%s' \n%srelations='%s' interfaces='%s'" \
-                  % (acc.name or 'None', acc.getRoleName(),
-                     state_string, indent, rel_string, iface_string)
-    except:
+    if AXObject.is_dead(acc):
         string += "(exception fetching data)"
+        return string
 
+    name_string = "name='%s'".replace("\n", "\\n") % AXObject.get_name(acc)
+    desc_string = "%sdescription='%s'".replace("\n", "\\n") % \
+        (indent, AXObject.get_description(acc))
+    role_string = f"role='{AXObject.get_role_name(acc)}'"
+    path_string = f"{indent}path={AXObject.get_path(acc)}"
+    state_string = f"{indent}states='{AXObject.state_set_as_string(acc)}'"
+    rel_string = f"{indent}relations='{AXObject.relations_as_string(acc)}'"
+    actions_string = f"{indent}actions='{AXObject.actions_as_string(acc)}'"
+    iface_string = f"{indent}interfaces='{AXObject.supported_interfaces_as_string(acc)}'"
+    attr_string = f"{indent}attributes='{AXObject.attributes_as_string(acc)}'"
+
+    string += "%s %s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n" \
+                  % (name_string, role_string, desc_string, state_string, rel_string,
+                     actions_string, iface_string, attr_string, path_string)
     return string
 
-
 # The following code originated from the following URL:
 # 
 # http://www.dalkescientific.com/writings/diary/archive/ \
@@ -389,7 +443,7 @@ def _getFileAndModule(frame):
     try:
         filename = frame.f_globals["__file__"]
         module = frame.f_globals["__name__"]
-    except:
+    except Exception:
         pass
     else:
         if (filename.endswith(".pyc") or filename.endswith(".pyo")):
@@ -403,16 +457,11 @@ def _shouldTraceIt():
 
     eventSource = objEvent.source
     if TRACE_APPS:
-        app = objEvent.host_application or eventSource.getApplication()
-        try:
-            app = objEvent.host_application or eventSource.getApplication()
-        except:
-            pass
-        else:
-            if not app.name in TRACE_APPS:
-                return False
+        app = AXObject.get_application(eventSource)
+        if AXObject.get_name(app) not in TRACE_APPS:
+            return False
 
-    if TRACE_ROLES and not eventSource.getRole() in TRACE_ROLES:
+    if TRACE_ROLES and AXObject.get_role(eventSource) not in TRACE_ROLES:
         return False
 
     if TRACE_EVENTS and \
@@ -423,9 +472,9 @@ def _shouldTraceIt():
 
 def traceit(frame, event, arg):
     """Line tracing utility to output all lines as they are executed by
-    the interpreter.  This is to be used by sys.settrace and is for 
+    the interpreter.  This is to be used by sys.settrace and is for
     debugging purposes.
-   
+
     Arguments:
     - frame: is the current stack frame
     - event: 'call', 'line', 'return', 'exception', 'c_call', 'c_return',
@@ -441,14 +490,14 @@ def traceit(frame, event, arg):
         return traceit
     if module in TRACE_IGNORE_MODULES:
         return traceit
-    if TRACE_MODULES and not module.split('.')[0] in TRACE_MODULES:
+    if TRACE_MODULES and module.split('.')[0] not in TRACE_MODULES:
         return traceit
-    if not event in ['call', 'line', 'return']:
+    if event not in ['call', 'line', 'return']:
         return traceit
 
     lineno = frame.f_lineno
     line = linecache.getline(filename, lineno).rstrip()
-    output = 'TRACE %s:%s: %s' % (module, lineno, line)
+    output = f'TRACE {module}:{lineno}: {line}'
 
     if event == 'call':
         argvals = inspect.getargvalues(frame)
@@ -461,13 +510,13 @@ def traceit(frame, event, arg):
             else:
                 return traceit
         for i, key in enumerate(keys):
-            output += '\n  ARG %s=%s' % (key, values[i])
+            output += f'\n  ARG {key}={values[i]}'
 
     lineElements = line.strip().split()
     if lineElements and lineElements[0] == 'return':
         if event == 'line':
             return traceit
-        output = '%s (rv: %s)' % (output, arg)
+        output = f'{output} (rv: {arg})'
 
     println(LEVEL_ALL, output)
 
@@ -482,50 +531,51 @@ def getOpenFDCount(pid):
 
 def getCmdline(pid):
     try:
-        openFile = os.popen('cat /proc/%s/cmdline' % pid)
+        openFile = os.popen(f'cat /proc/{pid}/cmdline')
         cmdline = openFile.read()
         openFile.close()
-    except:
+    except Exception:
         cmdline = '(Could not obtain cmdline)'
     cmdline = cmdline.replace('\x00', ' ')
 
     return cmdline
 
 def pidOf(procName):
-    openFile = subprocess.Popen('pgrep %s' % procName,
+    openFile = subprocess.Popen(f'pgrep {procName}',
                                 shell=True,
                                 stdout=subprocess.PIPE).stdout
     pids = openFile.read()
     openFile.close()
     return [int(p) for p in pids.split()]
 
-def examineProcesses():
-    desktop = pyatspi.Registry.getDesktop(0)
-    println(LEVEL_ALL, 'INFO: Desktop has %i apps:' % desktop.childCount)
-    for i, app in enumerate(desktop):
-        pid = app.get_process_id()
+def examineProcesses(force=False):
+    if force:
+        level = LEVEL_OFF
+    else:
+        level = LEVEL_ALL
+
+    desktop = AXUtilities.get_desktop()
+    println(level, 'INFO: Desktop has %i apps:' % AXObject.get_child_count(desktop), True)
+    for i, app in enumerate(AXObject.iter_children(desktop)):
+        pid = AXObject.get_process_id(app)
         cmd = getCmdline(pid)
         fds = getOpenFDCount(pid)
-        try:
-            name = app.name
-        except:
-            name = 'ERROR: Could not get name'
-        else:
-            if name == '':
-                name = 'WARNING: Possible hang'
-        println(LEVEL_ALL, '%3i. %s (pid: %s) %s file descriptors: %i' \
-                    % (i+1, name, pid, cmd, fds))
+        name = AXObject.get_name(app)
+        if name == '':
+            name = 'WARNING: Possible hang or dead app'
+        println(level, '%3i. %s (pid: %s) %s file descriptors: %i' \
+                    % (i+1, name, pid, cmd, fds), True)
 
     # Other 'suspect' processes which might not show up as accessible apps.
     otherApps = ['apport']
     for app in otherApps:
         pids = pidOf(app)
         if not pids:
-            println(LEVEL_ALL, 'INFO: no pid for %s' % app)
+            println(level, f'INFO: no pid for {app}', True)
             continue
 
         for pid in pids:
             cmd = getCmdline(pid)
             fds = getOpenFDCount(pid)
-            println(LEVEL_ALL, 'INFO: %s (pid: %s) %s file descriptors: %i' \
-                        % (app, pid, cmd, fds))
+            println(level, 'INFO: %s (pid: %s) %s file descriptors: %i' \
+                        % (app, pid, cmd, fds), True)
