@@ -33,45 +33,29 @@ import faulthandler
 import gi
 import importlib
 import os
-import pyatspi
 import re
 import signal
 import subprocess
 import sys
 
+gi.require_version("Atspi", "2.0")
+gi.require_version("Gdk", "3.0")
+from gi.repository import Atspi
+from gi.repository import Gdk
+
 try:
     from gi.repository.Gio import Settings
     a11yAppSettings = Settings(schema_id='org.gnome.desktop.a11y.applications')
-except:
+except Exception:
     a11yAppSettings = None
 
-try:
-    # This can fail due to gtk not being available.  We want to
-    # be able to recover from that if possible.  The main driver
-    # for this is to allow "orca --text-setup" to work even if
-    # the desktop is not running.
-    #
-    gi.require_version("Gtk", "3.0")
-    from gi.repository import Gtk
-
-    gi.require_version("Gdk", "3.0")
-    from gi.repository import Gdk
-
-    # Note: This last import is here due to bgo #673396.
-    # See bgo#673397 for the rest of the story.
-    gi.require_version("GdkX11", "3.0")
-    from gi.repository.GdkX11 import X11Screen
-except:
-    pass
-
 from . import braille
 from . import debug
 from . import event_manager
-from . import keybindings
+from . import learn_mode_presenter
 from . import logger
 from . import messages
 from . import mouse_review
-from . import notification_messages
 from . import orca_state
 from . import orca_platform
 from . import script_manager
@@ -79,6 +63,8 @@ from . import settings
 from . import settings_manager
 from . import speech
 from . import sound
+from .ax_object import AXObject
+from .ax_utilities import AXUtilities
 from .input_event import BrailleEvent
 
 _eventManager = event_manager.getManager()
@@ -89,7 +75,7 @@ _logger = logger.getLogger()
 def onEnabledChanged(gsetting, key):
     try:
         enabled = gsetting.get_boolean(key)
-    except:
+    except Exception:
         return
 
     if key == 'screen-reader-enabled' and not enabled:
@@ -121,6 +107,76 @@ _restoreOrcaKeys = False
 #                                                                      #
 ########################################################################
 
+CARET_TRACKING = "caret-tracking"
+FOCUS_TRACKING = "focus-tracking"
+FLAT_REVIEW = "flat-review"
+MOUSE_REVIEW = "mouse-review"
+OBJECT_NAVIGATOR = "object-navigator"
+SAY_ALL = "say-all"
+
+def getActiveModeAndObjectOfInterest():
+    tokens = ["ORCA: Active mode:", orca_state.activeMode,
+              "Object of interest:", orca_state.objOfInterest]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
+    return orca_state.activeMode, orca_state.objOfInterest
+
+def emitRegionChanged(obj, startOffset=None, endOffset=None, mode=None):
+    """Notifies interested clients that the current region of interest has changed."""
+
+    if startOffset is None:
+        startOffset = 0
+    if endOffset is None:
+        endOffset = startOffset
+    if mode is None:
+        mode = FOCUS_TRACKING
+
+    try:
+        obj.emit("mode-changed::" + mode, 1, "")
+    except Exception:
+        msg = "ORCA: Exception emitting mode-changed notification"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+    if mode != orca_state.activeMode:
+        tokens = ["ORCA: Switching active mode from", orca_state.activeMode, "to", mode]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        orca_state.activeMode = mode
+
+    try:
+        tokens = ["ORCA: Region of interest:", obj, "(", startOffset, ")", endOffset]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        obj.emit("region-changed", startOffset, endOffset)
+    except Exception:
+        msg = "ORCA: Exception emitting region-changed notification"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+    if obj != orca_state.objOfInterest:
+        tokens = ["ORCA: Switching object of interest from", orca_state.objOfInterest, "to", obj]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
+        orca_state.objOfInterest = obj
+
+def setActiveWindow(frame, app=None, alsoSetLocusOfFocus=False, notifyScript=False):
+    tokens = ["ORCA: Request to set active window to", frame]
+    if app is not None:
+        tokens.extend(["in", app])
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
+
+    if frame == orca_state.activeWindow:
+        msg = "ORCA: Setting activeWindow to existing activeWindow"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+    elif frame is None:
+        orca_state.activeWindow = None
+    else:
+        real_app, real_frame = AXObject.find_real_app_and_window_for(frame, app)
+        if real_frame != frame:
+            tokens = ["ORCA: Correcting active window to", real_frame, "in", real_app]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
+            orca_state.activeWindow = real_frame
+        else:
+            orca_state.activeWindow = frame
+
+    if alsoSetLocusOfFocus:
+        setLocusOfFocus(None, orca_state.activeWindow, notifyScript=notifyScript)
+
 def setLocusOfFocus(event, obj, notifyScript=True, force=False):
     """Sets the locus of focus (i.e., the object with visual focus) and
     notifies the script of the change should the script wish to present
@@ -136,41 +192,38 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
 
     if not force and obj == orca_state.locusOfFocus:
         msg = "ORCA: Setting locusOfFocus to existing locusOfFocus"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return
 
     if event and (orca_state.activeScript and not orca_state.activeScript.app):
-        script = _scriptManager.getScript(event.host_application, event.source)
+        app = AXObject.get_application(event.source)
+        script = _scriptManager.getScript(app, event.source)
         _scriptManager.setActiveScript(script, "Setting locusOfFocus")
 
     oldFocus = orca_state.locusOfFocus
-    try:
-        oldFocus.getRole()
-    except:
-        msg = "ORCA: Old locusOfFocus is null or defunct"
-        debug.println(debug.LEVEL_INFO, msg, True)
+    if AXObject.is_dead(oldFocus):
         oldFocus = None
 
-    if not obj:
+    if obj is None:
         msg = "ORCA: New locusOfFocus is null (being cleared)"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         orca_state.locusOfFocus = None
         return
 
     if orca_state.activeScript:
-        msg = "ORCA: Active script is: %s" % orca_state.activeScript
-        debug.println(debug.LEVEL_INFO, msg, True)
+        tokens = ["ORCA: Active script is:", orca_state.activeScript]
+        debug.printTokens(debug.LEVEL_INFO, tokens, True)
         if orca_state.activeScript.utilities.isZombie(obj):
-            msg = "ERROR: New locusOfFocus (%s) is zombie. Not updating." % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+            tokens = ["ERROR: New locusOfFocus (", obj, ") is zombie. Not updating."]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return
-        if orca_state.activeScript.utilities.isDead(obj):
-            msg = "ERROR: New locusOfFocus (%s) is dead. Not updating." % obj
-            debug.println(debug.LEVEL_INFO, msg, True)
+        if AXObject.is_dead(obj):
+            tokens = ["ERROR: New locusOfFocus (", obj, ") is dead. Not updating."]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             return
 
-    msg = "ORCA: Changing locusOfFocus from %s to %s" % (oldFocus, obj)
-    debug.println(debug.LEVEL_INFO, msg, True)
+    tokens = ["ORCA: Changing locusOfFocus from", oldFocus, "to", obj, ". Notify:", notifyScript]
+    debug.printTokens(debug.LEVEL_INFO, tokens, True)
     orca_state.locusOfFocus = obj
 
     if not notifyScript:
@@ -178,7 +231,7 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
 
     if not orca_state.activeScript:
         msg = "ORCA: Cannot notify active script because there isn't one"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         return
 
     orca_state.activeScript.locusOfFocusChanged(event, oldFocus, orca_state.locusOfFocus)
@@ -209,10 +262,11 @@ def _processBrailleEvent(event):
 
     try:
         consumed = _eventManager.processBrailleEvent(event)
-    except:
+    except Exception:
         debug.printException(debug.LEVEL_SEVERE)
 
-    if (not consumed) and orca_state.learnModeEnabled:
+    # TODO - JD: Is this still possible?
+    if not consumed and learn_mode_presenter.getPresenter().is_active():
         consumed = True
 
     return consumed
@@ -223,6 +277,15 @@ def _processBrailleEvent(event):
 #                                                                      #
 ########################################################################
 
+def deviceChangeHandler(deviceManager, device):
+    """New keyboards being plugged in stomp on our changes to the keymappings,
+       so we have to re-apply"""
+    source = device.get_source()
+    if source == Gdk.InputSource.KEYBOARD:
+        msg = "ORCA: Keyboard change detected, re-creating the xmodmap"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        _createOrcaXmodmap()
+
 def updateKeyMap(keyboardEvent):
     """Unsupported convenience method to call sad hacks which should go away."""
 
@@ -250,32 +313,51 @@ def _setCapsLockAsOrcaModifier(enable):
     """Enable or disable use of the caps lock key as an Orca modifier key."""
     interpretCapsLineProg = re.compile(
         r'^\s*interpret\s+Caps[_+]Lock[_+]AnyOfOrNone\s*\(all\)\s*{\s*$', re.I)
-    capsModLineProg = re.compile(
-        r'^\s*action\s*=\s*NoAction\s*\(\s*\)\s*;\s*$', re.I)
     normalCapsLineProg = re.compile(
         r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Lock\s*\)\s*;\s*$', re.I)
+    interpretShiftLineProg = re.compile(
+        r'^\s*interpret\s+Shift[_+]Lock[_+]AnyOf\s*\(\s*Shift\s*\+\s*Lock\s*\)\s*{\s*$', re.I)
+    normalShiftLineProg = re.compile(
+        r'^\s*action\s*=\s*LockMods\s*\(\s*modifiers\s*=\s*Shift\s*\)\s*;\s*$', re.I)
+    disabledModLineProg = re.compile(
+        r'^\s*action\s*=\s*NoAction\s*\(\s*\)\s*;\s*$', re.I)
     normalCapsLine = '        action= LockMods(modifiers=Lock);'
-    capsModLine =    '        action= NoAction();'
+    normalShiftLine = '        action= LockMods(modifiers=Shift);'
+    disabledModLine = '        action= NoAction();'
     lines = _originalXmodmap.decode('UTF-8').split('\n')
     foundCapsInterpretSection = False
+    foundShiftInterpretSection = False
+    modified = False
     for i, line in enumerate(lines):
-        if not foundCapsInterpretSection:
+        if not foundCapsInterpretSection and not foundShiftInterpretSection:
             if interpretCapsLineProg.match(line):
                 foundCapsInterpretSection = True
-        else:
+            elif interpretShiftLineProg.match(line):
+                foundShiftInterpretSection = True
+        elif foundCapsInterpretSection:
             if enable:
                 if normalCapsLineProg.match(line):
-                    lines[i] = capsModLine
-                    _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
-                    return
+                    lines[i] = disabledModLine
+                    modified = True
             else:
-                if capsModLineProg.match(line):
+                if disabledModLineProg.match(line):
                     lines[i] = normalCapsLine
-                    _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
-                    return
+                    modified = True
             if line.find('}'):
-                # Failed to find the line we need to change
-                return
+                foundCapsInterpretSection = False
+        else: # foundShiftInterpretSection
+            if enable:
+                if normalShiftLineProg.match(line):
+                    lines[i] = disabledModLine
+                    modified = True
+            else:
+                if disabledModLineProg.match(line):
+                    lines[i] = normalShiftLine
+                    modified = True
+            if line.find('}'):
+                foundShiftInterpretSection = False
+    if modified:
+        _setXmodmap(bytes('\n'.join(lines), 'UTF-8'))
 
 def _createOrcaXmodmap():
     """Makes an Orca-specific Xmodmap so that the keys behave as we
@@ -284,8 +366,8 @@ def _createOrcaXmodmap():
 
     global _capsLockCleared
 
-    cmd = []
-    if "Caps_Lock" in settings.orcaModifierKeys:
+    if "Caps_Lock" in settings.orcaModifierKeys \
+       or "Shift_Lock" in settings.orcaModifierKeys:
         _setCapsLockAsOrcaModifier(True)
         _capsLockCleared = True
     elif _capsLockCleared:
@@ -310,12 +392,23 @@ def _restoreXmodmap(keyList=[]):
       to restore the entire saved xmodmap.
     """
 
+    msg = "ORCA: Attempting to restore original xmodmap"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
     global _capsLockCleared
     _capsLockCleared = False
     p = subprocess.Popen(['xkbcomp', '-w0', '-', os.environ['DISPLAY']],
         stdin=subprocess.PIPE, stdout=None, stderr=None)
     p.communicate(_originalXmodmap)
 
+    msg = "ORCA: Original xmodmap restored"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
+def setKeyHandling(new):
+    """Toggle use of the new vs. legacy key handling mode.
+    """
+    _eventManager.setKeyHandling(new)
+
 def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     """Loads (and reloads) the user settings module, reinitializing
     things such as speech if necessary.
@@ -323,7 +416,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     Returns True to indicate the input event has been consumed.
     """
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Loading User Settings', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Loading User Settings', True)
 
     global _userSettings
 
@@ -345,7 +438,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
             reloaded = True
         except ImportError:
             debug.printException(debug.LEVEL_INFO)
-        except:
+        except Exception:
             debug.printException(debug.LEVEL_SEVERE)
     else:
         _profile = _settingsManager.profile
@@ -353,7 +446,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
             _userSettings = _settingsManager.getGeneralSettings(_profile)
         except ImportError:
             debug.printException(debug.LEVEL_INFO)
-        except:
+        except Exception:
             debug.printException(debug.LEVEL_SEVERE)
 
     if not script:
@@ -362,28 +455,36 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     _settingsManager.loadAppSettings(script)
 
     if _settingsManager.getSetting('enableSpeech'):
+        msg = 'ORCA: About to enable speech'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         try:
             speech.init()
             if reloaded and not skipReloadMessage:
                 script.speakMessage(messages.SETTINGS_RELOADED)
-        except:
+        except Exception:
             debug.printException(debug.LEVEL_SEVERE)
     else:
         msg = 'ORCA: Speech is not enabled in settings'
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     if _settingsManager.getSetting('enableBraille'):
+        msg = 'ORCA: About to enable braille'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         try:
-            braille.init(_processBrailleEvent, settings.tty)
-        except:
+            braille.init(_processBrailleEvent)
+        except Exception:
             debug.printException(debug.LEVEL_WARNING)
             msg = 'ORCA: Could not initialize connection to braille.'
-            debug.println(debug.LEVEL_WARNING, msg, True)
+            debug.printMessage(debug.LEVEL_WARNING, msg, True)
+    else:
+        msg = 'ORCA: Braille is not enabled in settings'
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+
 
     if _settingsManager.getSetting('enableMouseReview'):
-        mouse_review.reviewer.activate()
+        mouse_review.getReviewer().activate()
     else:
-        mouse_review.reviewer.deactivate()
+        mouse_review.getReviewer().deactivate()
 
     if _settingsManager.getSetting('enableSound'):
         player.init()
@@ -402,7 +503,7 @@ def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     _scriptManager.activate()
     _eventManager.activate()
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: User Settings Loaded', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: User Settings Loaded', True)
 
     return True
 
@@ -413,7 +514,7 @@ def _showPreferencesUI(script, prefs):
 
     try:
         module = importlib.import_module('.orca_gui_prefs', 'orca')
-    except:
+    except Exception:
         debug.printException(debug.LEVEL_SEVERE)
         return
 
@@ -456,19 +557,30 @@ def showPreferencesGUI(script=None, inputEvent=None):
 
     return True
 
-def helpForOrca(script=None, inputEvent=None, page=""):
-    """Show Orca Help window (part of the GNOME Access Guide).
+def addKeyGrab(binding):
+    """ Add a key grab for the given key binding."""
 
-    Returns True to indicate the input event has been consumed.
-    """
-    orca_state.learnModeEnabled = False
-    uri = "help:orca"
-    if page:
-        uri += "?%s" % page
-    Gtk.show_uri(Gdk.Screen.get_default(),
-                 uri,
-                 Gtk.get_current_event_time())
-    return True
+    if orca_state.device is None:
+        return []
+
+    ret = []
+    for kd in binding.keyDefs():
+        ret.append(orca_state.device.add_key_grab(kd, None))
+    return ret
+
+def removeKeyGrab(id):
+    """ Remove the key grab for the given key binding."""
+
+    if orca_state.device is None:
+        return
+
+    orca_state.device.remove_key_grab(id)
+
+def mapModifier(keycode):
+    if orca_state.device is None:
+        return
+
+    return orca_state.device.map_modifier(keycode)
 
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
@@ -490,14 +602,14 @@ def showFindGUI(script=None, inputEvent=None):
     try:
         module = importlib.import_module('.orca_gui_find', 'orca')
         module.showFindUI()
-    except:
+    except Exception:
         debug.printException(debug.LEVEL_SEVERE)
 
 # If True, this module has been initialized.
 #
 _initialized = False
 
-def init(registry):
+def init():
     """Initialize the orca module, which initializes the speech and braille
     modules.  Also builds up the application list, registers for AT-SPI events,
     and creates scripts for all known applications.
@@ -506,11 +618,12 @@ def init(registry):
     module has already been initialized.
     """
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Initializing', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Initializing', True)
 
     global _initialized
 
     if _initialized and _settingsManager.isScreenReaderServiceEnabled():
+        debug.printMessage(debug.LEVEL_INFO, 'ORCA: Already initialized', True)
         return False
 
     # Do not hang on initialization if we can help it.
@@ -531,18 +644,17 @@ def init(registry):
     if a11yAppSettings:
         a11yAppSettings.connect('changed', onEnabledChanged)
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Initialized', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Initialized', True)
 
     return True
 
-def start(registry, cacheValues):
-    """Starts Orca.
-    """
+def start():
+    """Starts Orca."""
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Starting', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Starting', True)
 
     if not _initialized:
-        init(registry)
+        init()
 
     # Do not hang on startup if we can help it.
     #
@@ -553,15 +665,21 @@ def start(registry, cacheValues):
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
 
-    if cacheValues:
-        pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
+    # Event handlers for input devices being plugged in/unplugged.
+    # Used to re-create the Xmodmap when a new keyboard is plugged in.
+    # Necessary, because plugging in a new keyboard resets the Xmodmap
+    # and stomps our changes
+    display = Gdk.Display.get_default()
+    devmanager=display.get_device_manager()
+    devmanager.connect("device-added", deviceChangeHandler)
+    devmanager.connect("device-removed", deviceChangeHandler)
 
     Gdk.notify_startup_complete()
     msg = 'ORCA: Startup complete notification made'
-    debug.println(debug.LEVEL_INFO, msg, True)
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Starting registry', True)
-    registry.start(gil=False)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Starting Atspi main event loop', True)
+    Atspi.event_main()
 
 def die(exitCode=1):
     pid = os.getpid()
@@ -577,9 +695,9 @@ def die(exitCode=1):
 
 def timeout(signum=None, frame=None):
     msg = 'TIMEOUT: something has hung. Aborting.'
-    debug.println(debug.LEVEL_SEVERE, msg, True)
-    debug.printStack(debug.LEVEL_ALL)
-    debug.examineProcesses()
+    debug.printMessage(debug.LEVEL_SEVERE, msg, True)
+    debug.printStack(debug.LEVEL_SEVERE)
+    debug.examineProcesses(force=True)
     die(EXIT_CODE_HANG)
 
 def shutdown(script=None, inputEvent=None):
@@ -589,7 +707,7 @@ def shutdown(script=None, inputEvent=None):
     was never initialized.
     """
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Shutting down', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Shutting down', True)
 
     global _initialized
 
@@ -602,10 +720,13 @@ def shutdown(script=None, inputEvent=None):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
-    orca_state.activeScript.presentMessage(messages.STOP_ORCA)
+    orca_state.activeScript.presentationInterrupt()
+    orca_state.activeScript.presentMessage(messages.STOP_ORCA, resetStyles=False)
 
-    _scriptManager.deactivate()
+    # Deactivate the event manager first so that it clears its queue and will not
+    # accept new events. Then let the script manager unregister script event listeners.
     _eventManager.deactivate()
+    _scriptManager.deactivate()
 
     # Shutdown all the other support.
     #
@@ -623,10 +744,9 @@ def shutdown(script=None, inputEvent=None):
     _initialized = False
     _restoreXmodmap(_orcaModifiers)
 
-    debug.println(debug.LEVEL_INFO, 'ORCA: Stopping registry', True)
-    pyatspi.Registry.stop()
-
-    debug.println(debug.LEVEL_INFO, 'ORCA: Shutdown complete', True)
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Quitting Atspi main event loop', True)
+    Atspi.event_quit()
+    debug.printMessage(debug.LEVEL_INFO, 'ORCA: Shutdown complete', True)
 
     return True
 
@@ -634,8 +754,9 @@ exitCount = 0
 def shutdownOnSignal(signum, frame):
     global exitCount
 
-    msg = 'ORCA: Shutting down and exiting due to signal=%d' % signum
-    debug.println(debug.LEVEL_INFO, msg, True)
+    signalString = f'({signal.strsignal(signum)})'
+    msg = f"ORCA: Shutting down and exiting due to signal={signum} {signalString}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
 
     # Well...we'll try to exit nicely, but if we keep getting called,
     # something bad is happening, so just quit.
@@ -661,7 +782,7 @@ def shutdownOnSignal(signum, frame):
             speech.shutdown()
             shutdown()
         cleanExit = True
-    except:
+    except Exception:
         cleanExit = False
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
@@ -671,19 +792,38 @@ def shutdownOnSignal(signum, frame):
         die(EXIT_CODE_HANG)
 
 def crashOnSignal(signum, frame):
-    signal.signal(signum, signal.SIG_DFL)
+    signalString = f'({signal.strsignal(signum)})'
+    msg = f"ORCA: Shutting down and exiting due to signal={signum} {signalString}"
+    debug.printMessage(debug.LEVEL_SEVERE, msg, True)
+    debug.printStack(debug.LEVEL_SEVERE)
     _restoreXmodmap(_orcaModifiers)
-    os.kill(os.getpid(), signum)
+    try:
+        orca_state.activeScript.presentationInterrupt()
+        orca_state.activeScript.presentMessage(messages.STOP_ORCA, resetStyles=False)
+    except Exception:
+        pass
+    sys.exit(1)
 
-def main(cacheValues=True):
+def main():
     """The main entry point for Orca.  The exit codes for Orca will
     loosely be based on signals, where the exit code will be the
     signal used to terminate Orca (if a signal was used).  Otherwise,
     an exit code of 0 means normal completion and an exit code of 50
     means Orca exited because of a hang."""
 
+    msg = f"ORCA: Launching version {orca_platform.version}"
+    if orca_platform.revision:
+        msg += f" (rev {orca_platform.revision})"
+
+    sessionType = os.environ.get('XDG_SESSION_TYPE') or ""
+    sessionDesktop = os.environ.get('XDG_SESSION_DESKTOP') or ""
+    session = "%s %s".strip() % (sessionType, sessionDesktop)
+    if session:
+        msg += f" session: {session}"
+    debug.printMessage(debug.LEVEL_INFO, msg, True)
+
     if debug.debugFile and os.path.exists(debug.debugFile.name):
-        faulthandler.enable(file=debug.debugFile, all_threads=False)
+        faulthandler.enable(file=debug.debugFile, all_threads=True)
     else:
         faulthandler.enable(all_threads=False)
 
@@ -699,38 +839,51 @@ def main(cacheValues=True):
     signal.signal(signal.SIGQUIT, shutdownOnSignal)
     signal.signal(signal.SIGSEGV, crashOnSignal)
 
+    debug.printMessage(debug.LEVEL_INFO, "ORCA: Enabling accessibility (if needed).", True)
     if not _settingsManager.isAccessibilityEnabled():
         _settingsManager.setAccessibility(True)
 
-    init(pyatspi.Registry)
+    debug.printMessage(debug.LEVEL_INFO, "ORCA: Initializing.", True)
+    init()
+    debug.printMessage(debug.LEVEL_INFO, "ORCA: Initialized.", True)
 
     try:
         message = messages.START_ORCA
         script = _scriptManager.getDefaultScript()
         script.presentMessage(message)
-    except:
+    except Exception:
         debug.printException(debug.LEVEL_SEVERE)
 
     script = orca_state.activeScript
     if script:
         window = script.utilities.activeWindow()
+
         if window and not orca_state.locusOfFocus:
-            try:
-                app = window.getApplication()
-            except:
-                msg = "ORCA: Exception getting app for %s" % window
-                debug.println(debug.LEVEL_INFO, msg, True)
-            else:
-                script = _scriptManager.getScript(app, window)
-                _scriptManager.setActiveScript(script, "Launching.")
-            setLocusOfFocus(None, window)
-            focusedObject = script.utilities.focusedObject(window)
+            app = AXObject.get_application(window)
+            setActiveWindow(window, app, alsoSetLocusOfFocus=True, notifyScript=True)
+
+            # setActiveWindow does some corrective work needed thanks to
+            # mutter-x11-frames. So retrieve the window just in case.
+            window = orca_state.activeWindow
+            script = _scriptManager.getScript(app, window)
+            _scriptManager.setActiveScript(script, "Launching.")
+
+            focusedObject = AXUtilities.get_focused_object(window)
+            tokens = ["ORCA: Focused object is:", focusedObject]
+            debug.printTokens(debug.LEVEL_INFO, tokens, True)
             if focusedObject:
                 setLocusOfFocus(None, focusedObject)
+                script = _scriptManager.getScript(
+                    AXObject.get_application(focusedObject), focusedObject)
+                _scriptManager.setActiveScript(script, "Found focused object.")
 
     try:
-        start(pyatspi.Registry, cacheValues) # waits until we stop the registry
-    except:
+        msg = "ORCA: Starting ATSPI registry."
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        start() # waits until we stop the registry
+    except Exception:
+        msg = "ORCA: Exception starting ATSPI registry."
+        debug.printMessage(debug.LEVEL_SEVERE, msg, True)
         die(EXIT_CODE_HANG)
     return 0
 
