@@ -27,11 +27,14 @@ __copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc." \
                 "Copyright (c) 2014-2015 Igalia, S.L."
 __license__   = "LGPL"
 
-import pyatspi
+import gi
+gi.require_version("Atspi", "2.0")
+from gi.repository import Atspi
 
 from orca import debug
 from orca import orca
 from orca import orca_state
+from orca.ax_object import AXObject
 from orca.scripts import default
 from orca.scripts import web
 from .script_utilities import Utilities
@@ -42,17 +45,10 @@ class Script(web.Script):
     def __init__(self, app):
         super().__init__(app)
 
-        # This can be removed once this Mozilla bug is fixed:
-        # https://bugzilla.mozilla.org/show_bug.cgi?id=1174204
-        self.attributeNamesDict = {
-            "background-color"        : "bg-color",
-            "color"                   : "fg-color"}
-
-        # This one needs some more consideration for all toolkits.
-        self.attributeNamesDict["invalid"] = "text-spelling"
+        self.presentIfInactive = False
 
     def getUtilities(self):
-        """Returns the utilites for this script."""
+        """Returns the utilities for this script."""
 
         return Utilities(self)
 
@@ -69,7 +65,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.locusOfFocusChanged(self, event, oldFocus, newFocus)
 
     def onActiveChanged(self, event):
@@ -78,12 +74,12 @@ class Script(web.Script):
         if super().onActiveChanged(event):
             return
 
-        if event.detail1 and event.source.getRole() == pyatspi.ROLE_FRAME \
+        if event.detail1 and AXObject.get_role(event.source) == Atspi.Role.FRAME \
            and not self.utilities.canBeActiveWindow(event.source):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onActiveChanged(self, event)
 
     def onActiveDescendantChanged(self, event):
@@ -93,20 +89,22 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onActiveDescendantChanged(self, event)
 
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
         if self.utilities.isNotRealDocument(event.source):
+            msg = "GECKO: Ignoring: Event source is not real document"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         if super().onBusyChanged(event):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onBusyChanged(self, event)
 
     def onCaretMoved(self, event):
@@ -116,7 +114,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onCaretMoved(self, event)
 
     def onCheckedChanged(self, event):
@@ -126,30 +124,52 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onCheckedChanged(self, event)
 
-    def onChildrenChanged(self, event):
-        """Callback for object:children-changed accessibility events."""
+    def onColumnReordered(self, event):
+        """Callback for object:column-reordered accessibility events."""
 
-        if super().onChildrenChanged(event):
+        if super().onColumnReordered(event):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
-        default.Script.onChildrenChanged(self, event)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onColumnReordered(self, event)
+
+    def onChildrenAdded(self, event):
+        """Callback for object:children-changed:add accessibility events."""
+
+        if super().onChildrenAdded(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onChildrenAdded(self, event)
+
+    def onChildrenRemoved(self, event):
+        """Callback for object:children-changed:removed accessibility events."""
+
+        if super().onChildrenRemoved(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onChildrenRemoved(self, event)
 
     def onDocumentLoadComplete(self, event):
         """Callback for document:load-complete accessibility events."""
 
         if self.utilities.isNotRealDocument(event.source):
+            msg = "GECKO: Ignoring: Event source is not real document"
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         if super().onDocumentLoadComplete(event):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onDocumentLoadComplete(self, event)
 
     def onDocumentLoadStopped(self, event):
@@ -159,7 +179,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onDocumentLoadStopped(self, event)
 
     def onDocumentReload(self, event):
@@ -169,14 +189,26 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onDocumentReload(self, event)
 
+    def onExpandedChanged(self, event):
+        """Callback for object:state-changed:expanded accessibility events."""
+
+        if super().onExpandedChanged(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onExpandedChanged(self, event)
+
     def onFocus(self, event):
         """Callback for focus: accessibility events."""
 
-        # We should get proper state-changed events for these.
-        if self.utilities.inDocumentContent(event.source):
+        # This event is deprecated. We should get object:state-changed:focused
+        # events instead.
+
+        if super().onFocus(event):
             return
 
         if self.utilities.isLayoutOnly(event.source):
@@ -184,7 +216,7 @@ class Script(web.Script):
 
         if event.source == orca_state.activeWindow:
             msg = "GECKO: Ignoring event for active window."
-            debug.println(debug.LEVEL_INFO, msg, True)
+            debug.printMessage(debug.LEVEL_INFO, msg, True)
             return
 
         # NOTE: This event type is deprecated and Orca should no longer use it.
@@ -199,14 +231,14 @@ class Script(web.Script):
         if super().onFocusedChanged(event):
             return
 
-        if event.source.getRole() == pyatspi.ROLE_PANEL:
+        if AXObject.get_role(event.source) == Atspi.Role.PANEL:
             if orca_state.locusOfFocus == orca_state.activeWindow:
                 msg = "GECKO: Ignoring event believed to be noise."
-                debug.println(debug.LEVEL_INFO, msg, True)
+                debug.printMessage(debug.LEVEL_INFO, msg, True)
                 return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onFocusedChanged(self, event)
 
     def onMouseButton(self, event):
@@ -216,7 +248,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onMouseButton(self, event)
 
     def onNameChanged(self, event):
@@ -226,9 +258,19 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onNameChanged(self, event)
 
+    def onRowReordered(self, event):
+        """Callback for object:row-reordered accessibility events."""
+
+        if super().onRowReordered(event):
+            return
+
+        msg = "GECKO: Passing along event to default script"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onRowReordered(self, event)
+
     def onSelectedChanged(self, event):
         """Callback for object:state-changed:selected accessibility events."""
 
@@ -236,7 +278,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onSelectedChanged(self, event)
 
     def onSelectionChanged(self, event):
@@ -246,7 +288,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onSelectionChanged(self, event)
 
     def onShowingChanged(self, event):
@@ -255,15 +297,19 @@ class Script(web.Script):
         if super().onShowingChanged(event):
             return
 
-        if event.detail1 and self.utilities.isTopLevelChromeAlert(event.source):
-            msg = "GECKO: Event handled: Presenting event source"
-            debug.println(debug.LEVEL_INFO, msg, True)
-            self.presentObject(event.source)
-            return True
+        msg = "GECKO: Passing along event to default script"
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onShowingChanged(self, event)
+
+    def onTextAttributesChanged(self, event):
+        """Callback for object:text-attributes-changed accessibility events."""
+
+        if super().onTextAttributesChanged(event):
+            return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
-        default.Script.onShowingChanged(self, event)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
+        default.Script.onTextAttributesChanged(self, event)
 
     def onTextDeleted(self, event):
         """Callback for object:text-changed:delete accessibility events."""
@@ -272,7 +318,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onTextDeleted(self, event)
 
     def onTextInserted(self, event):
@@ -282,7 +328,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onTextInserted(self, event)
 
     def onTextSelectionChanged(self, event):
@@ -292,7 +338,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onTextSelectionChanged(self, event)
 
     def onWindowActivated(self, event):
@@ -305,7 +351,7 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onWindowActivated(self, event)
 
     def onWindowDeactivated(self, event):
@@ -315,5 +361,5 @@ class Script(web.Script):
             return
 
         msg = "GECKO: Passing along event to default script"
-        debug.println(debug.LEVEL_INFO, msg, True)
+        debug.printMessage(debug.LEVEL_INFO, msg, True)
         default.Script.onWindowDeactivated(self, event)
