@@ -27,8 +27,6 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2004-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import pyatspi
-
 from . import messages
 from .acss import ACSS
 
@@ -303,6 +301,9 @@ presentLockingKeys           = None
 # Mouse review
 enableMouseReview          = False
 
+# Flat review
+flatReviewIsRestricted = False
+
 # Progressbars
 speakProgressBarUpdates    = True
 brailleProgressBarUpdates  = False
